<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 */


namespace Amasty\Coupons\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\SalesRule\Model\CouponFactory;
use Magento\SalesRule\Model\ResourceModel\Coupon;
use Magento\SalesRule\Model\ResourceModel\Coupon\Usage;
use Amasty\Coupons\Model\CouponRenderer;
use Magento\Framework\Event\Observer;
use Magento\Sales\Model\Order;
use Magento\Framework\Exception\AlreadyExistsException;

/**
 * Class UpdateCouponUsage
 */
class UpdateCouponUsage implements ObserverInterface
{
    /**
     * @var Coupon
     */
    private $coupon;

    /**
     * @var Usage
     */
    private $couponUsage;

    /**
     * @var CouponRenderer
     */
    private $couponRenderer;

    /**
     * Save used coupon code ID
     *
     * @var
     */
    private $usedCodes = [];

    /**
     * @var CouponFactory
     */
    private $couponFactory;

    public function __construct(
        Coupon $coupon,
        Usage $couponUsage,
        CouponRenderer $couponRenderer,
        CouponFactory $couponFactory
    ) {
        $this->coupon = $coupon;
        $this->couponUsage = $couponUsage;
        $this->couponRenderer = $couponRenderer;
        $this->couponFactory = $couponFactory;
    }

    /**
     * event sales_order_place_after
     *
     * @param Observer $observer
     *
     * @throws AlreadyExistsException
     * @return $this
     */
    public function execute(Observer $observer)
    {
        /** @var Order $order */
        $order = $observer->getEvent()->getOrder();
        if (!$order) {
            return $this;
        }
        // if order placement then increment else if order cancel then decrement
        $increment = (bool)$observer->getEventName() !== 'order_cancel_after';
        $customerId = $order->getCustomerId();
        $coupons = $this->couponRenderer->parseCoupon($order->getCouponCode());
        if (is_array($coupons) && count($coupons) > 1) {
            foreach ($coupons as $coupon) {
                if ($this->isUsed($coupon)) {
                    continue;
                }

                /** @var \Magento\SalesRule\Model\Coupon $couponEntity */
                $couponEntity = $this->couponFactory->create();
                $this->coupon->load($couponEntity, $coupon, 'code');

                if ($couponEntity->getId()) {
                    $couponEntity->setTimesUsed($couponEntity->getTimesUsed() + ($increment ? 1 : -1));
                    $this->coupon->save($couponEntity);
                    if ($customerId) {
                        $this->couponUsage->updateCustomerCouponTimesUsed(
                            $customerId,
                            $couponEntity->getId(),
                            $increment
                        );
                    }
                }
            }
        }

        return $this;
    }

    /**
     * @param string $code
     *
     * @return bool
     */
    private function isUsed($code)
    {
        if (!isset($this->usedCodes[$code])) {
            $this->usedCodes[$code] = 1;

            return false;
        }

        return true;
    }
}
